set saveimagedir ""
set saveimagelist ""
set oldsaveformat TIFF
set oldsavegformat AVW
set oldtransform IntoTalairachSpace

set SaveAsImageList {TIFF Raw AVW AVW_SPM_TAL VEST EPS JPEG}
set SaveAsGroupList {AVW}
set TransformList {IntoTalairachSpace GenericReslice GenericResample MotionCorrectionReslice AlignWarpReslice CopyVOIReslice UserTransformation} 

proc OpenFolder { w } {
    SelectFile $w "Open Folder" "Open" OpenFolder FolderDirHistory
}

proc SaveImageAs { w } {
    SelectFile $w "Save Image As" "Save" SaveImageAs DirectoryHistory
}

proc SaveGroupAs { w } {
    SelectFile $w "Save Group As" "Save" SaveGroupAs DirectoryHistory
}

proc SaveFolderAs { w } { 
    SelectFile $w "Save Folder As" "Save" SaveFolderAs FolderDirHistory
}

proc ApplyParadigm {w} {
    SelectFile $w "Apply Paradigm" "Ok" ApplyParadigm DirectoryHistory
}

proc ApplyShadowTransform {w} {
    SelectFile $w "Apply Shadow Transform" "Ok" ApplyShadowTransform DirectoryHistory
}

proc SaveShadowTransform {w} {
    SelectFile $w "Save Shadow Transform" "Save" SaveShadowTransform DirectoryHistory
}

proc LoadROITemplate {w} {
    SelectFile $w "Load ROI Template" "Load" LoadROITemplate DirectoryHistory
}

proc SaveROITemplate {w} {
    SelectFile $w "Save ROI Template" "Save" SaveROITemplate DirectoryHistory
}

proc ReadEPSF {w} {
    SelectFile $w "Read EPSF" "Read" ReadEPSF DirectoryHistory
}

proc SelectHeadHat {w} {
    SelectFile $w "Select Head Hat File" "Select" SelectHeadHat DirectoryHistory
}

proc SaveGroupAsMenu {w} {
    $w add command AVW -label "AVW" 
    $w add command AVW_SPM_TAL -label "AVW (MEDx Tal To SPM)" 
}

# Procedure:	GetHistoryProperties
#
# Usage:	GetHistoryProperties widget target property
#
# Summary:	Retrieve property from target
#
# ie GetHistoryProperties .foo Session ImportImageHistory
#
# The widget here is the file box itself!
#
proc GetHistoryProperties { w target property size } {
    global $property

    set histlist [$w subwidget filter ] 

    Message:GetProperties DEFAULTS.$target $property prop
    set list [ PxGet $prop $property "./*" ]

    if { [llength $list ] == 0 } {
	set dir "~"
	set pattern "*"
    } else {
	# Use directory and pattern from the string, otherwise
	# the box always puts /* at the end of the string.
	regexp "(.*)/(.*)" [lindex $list 0 ] m dir pattern
	foreach i $list {
	    $histlist appendhistory $i
	}
    }

    $w config -directory $dir -pattern $pattern
}


# Procedure:	SetHistoryProperties
#
# Usage:	SetHistoryProperties widget target property size
#
# Summary:	Set property in target
# ie SetHistoryProperties .foo Session ImportImageHistory
proc SetHistoryProperties { w target property } { 
    global $property
    # convert a list to an array of strings
    set pxlist ""

    set plist [GetListItems $w]

    # cut list down to requested size
    set len [ min 10 [ llength $plist ] ]
    set plist [ lrange $plist 0 $len ]
    
    foreach dir "$plist" {
#	set dir [string trimright $dir "/*"]
	set pxlist "$pxlist \($dir\)"
    }

    Message:SetProperties DEFAULTS.$target "
        $property \[ $pxlist \]"
}

# luniq - remove duplicates from the list
proc luniq { list } {
    set newlist { }

    # search for every entry in the list, extract it, and put it
    # in the new list.
    foreach i $list {
	lappend $newlist [lindex $list [ lsearch -exact $list $i ]]
    }
    return $newlist
}

# Procedure:	SelectFile
#
# Usage:	SelectFile widget remotefileid filedialogtype
#
# Summary:	
#
proc SelectFile {w type okbuttontext help history } {
    global selectfile 
    global saveimagedir
    global saveimagelist
    global oldsaveformat oldsavegformat oldtransform currenttype
    global SaveAsImageList SaveAsGroupList TransformList

    if [winfo exists $w] {
	wm deiconify $w
	raise $w
	Message:DialogPosted $w
	return
    }

    set currenttype $type
    set selectfile(history) $history

    # Remember this funcname comes from remove blanks from the type
    # and the apply function is based on that.
    regsub -all " " $type "" selectfile($w,funcname)

    toplevel $w 
    wm title $w $type
    wm iconname $w $type

    # ---- Box ----

if { 0 } {
    if { $saveimagedir == "" } {
	set dir ""
	set pattern "*"
    } else {
	regexp "(.*)/(.*)" $saveimagedir m dir pattern
    }
}
    if { $type == "Open Folder" || $type == "Save Folder As"} {
        tixFolderSelectBox $w.box \
	    -command "SelectFile:doit $w"
#	    -directory $dir
    } else {
        tixFileSelectBox $w.box \
	    -command "SelectFile:doit $w"
#	    -directory $dir
    }

    #------------- Button Frame --------------

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
    button $w.ok -command "SelectFile:ok $w" \
	-text $okbuttontext -width 5
    bind $w.ok <Return> "$w.box invoke"
    button $w.apply -text "Apply" -width 5 \
	-command "SelectFile:ap $w" 
    bind $w.apply <Return> "$w.apply invoke"
    button $w.filter -command "$w.box filter" \
	-text "Filter" -width 5
    bind $w.filter <Return> {
	[winfo toplevel %W].box filter}
    button $w.cancel -command "DestroyDialog SelectFile $w" \
	-text "Cancel" -width 5
    bind $w.cancel <Return> {
	DestroyDialog SelectFile [winfo toplevel %W]}
    button $w.help   -command "WebHelp dialogs/$help" \
	-text "Help" -width 5

    pack $w.btns.b -fill x -padx 5 -pady 10
    pack $w.ok $w.filter $w.apply $w.cancel $w.help -in $w.btns.b \
	-side left -expand yes -padx 5 -pady 10 -fill y

    GetHistoryProperties $w.box Session $history 10

    #------------- Upper Frame --------------

    if {$type == "Save Image As"} {
	wm geometry $w 500x510
	frame $w.uf
	tixOptionMenu $w.uf.o -label "File Format" \
	    -variable selectfile($w,format) \
	    -options {
		label.width 12
		label.anchor w
		menubutton.width 20
	}
	SaveImageAsMenu $w.uf.o 
	
	if { $oldsaveformat == "" } {
	    set oldsaveformat TIFF
	}

	set validformat [lsearch -exact $SaveAsImageList $oldsaveformat]
	if { $validformat == -1 } {
	    set oldsaveformat TIFF
	}

	set selectfile($w,format) $oldsaveformat

	$w.uf.o configure -value $selectfile($w,format)

	$w.box.f2 paneconfigure directory -size 250

	set histlist [$w.box subwidget filter] 
	foreach i $saveimagelist {
	    set i [string trimright $i "*"]
	    set i [string trimright $i "/"]
	    $histlist appendhistory $i
	}

	pack $w.uf -pady 3 -fill x
	pack $w.uf.o -side top -padx 3 -pady 3
	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,disposition) ""

    } elseif {$type == "Save Group As"} {
	wm geometry $w 500x510
	frame $w.uf
	tixOptionMenu $w.uf.o -label "File Format" \
	    -variable selectfile($w,format) \
	    -options {
		label.width 12
		label.anchor w
		menubutton.width 20
	}
	SaveGroupAsMenu $w.uf.o 
	
	if { $oldsavegformat == "" } {
	    set oldsavegformat AVW
	}

	set validgformat [lsearch -exact $SaveAsGroupList $oldsavegformat]
	if { $validgformat == -1 } {
	    set oldsavegformat AVW
	}

	set selectfile($w,format) $oldsavegformat

	$w.uf.o configure -value $selectfile($w,format)

	$w.box.f2 paneconfigure directory -size 250

	set histlist [$w.box subwidget filter] 
	foreach i $saveimagelist {
	    set i [string trimright $i "*"]
	    set i [string trimright $i "/"]
	    $histlist appendhistory $i
	}

	pack $w.uf -pady 3 -fill x
	pack $w.uf.o -side top -padx 3 -pady 3
	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,disposition) ""

    } elseif {$type == "Open As Group"} {
	wm geometry $w 500x500

	$w.box.f2 paneconfigure directory -size 250

	set filewidget [ $w.box subwidget filelist subwidget listbox ]
	    $filewidget configure -selectmode extended

	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,format) ""
	set selectfile($w,disposition) ""

    } elseif {$type == "Apply Shadow Transform"} {
	wm geometry $w 500x450
	frame $w.uf
	tixOptionMenu $w.uf.o -label "Transform" \
	    -variable selectfile($w,format) \
	    -options {
		label.width 12
		label.anchor w
		menubutton.width 20
	}

        $w.uf.o add command IntoTalairachSpace -label "IntoTalairachSpace"
        $w.uf.o add command GenericReslice -label "GenericReslice"
        $w.uf.o add command GenericResample -label "GenericResample"
        $w.uf.o add command MotionCorrectionReslice -label "AlignLinearReslice"
        $w.uf.o add command AlignWarpReslice -label "AlignWarpReslice"
        $w.uf.o add command CopyVOIReslice -label "CopyVOIReslice"
        $w.uf.o add command UserTransformation -label "UserTransformation"
	
	if { $oldtransform == "" } {
	    set oldtransform IntoTalairachSpace
	}

	set validtransform [lsearch -exact $TransformList $oldtransform]
	if { $validtransform == -1 } {
	    set oldtransform IntoTalairachSpace
	}

	set selectfile($w,format) $oldtransform
	$w.uf.o configure -value $selectfile($w,format)

	$w.box.f2 paneconfigure directory -size 250

	pack $w.uf -pady 3 -fill x
	pack $w.uf.o -side top -padx 3 -pady 3
	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,disposition) ""

    } elseif {$type == "Save Folder As"} {
	wm geometry $w 500x450

	$w.box.f2 paneconfigure directory -size 250

	set selectfile($w,format) ""
	set selectfile($w,disposition) ""

	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.btns -fill both -pady 3
	tixBalloon $w.bhelp 

    } else {
	wm geometry $w 500x450
	$w.box.f2 paneconfigure directory -size 250

	set selectfile($w,format) ""
	set selectfile($w,disposition) ""

	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.btns -fill both
    }



    bind $w <Control-c> {
	DestroyDialog SelectFile [winfo toplevel %W]
    }

    Message:DialogPosted $w
}


# Procedure:	SelectFile:doit
#
# Usage:	SelectFile:doit widget filename
#
# Summary:	Invoke callback for fileSelectBox
#
proc SelectFile:doit {w filename} {
    global saveimage

    SelectFile:apply $w $filename
}

# Procedure:	SelectFile:ok
#
# Usage:	SelectFile:ok widget
#
# Summary:	Apply callback: sends the contents of the dialog box to the
#		remote file id.
#
proc SelectFile:ok {w} {
    global saveimage

    Message:OnNextCompletion "DestroyDialog SelectFile $w"
    $w.box invoke
}


# Procedure:	SelectFile:ap
#
# Usage:	SelectFile:ap widget filename
#
# Summary:	Apply callback: sends the contents of the dialog box to the
#		remote file id.
#
proc SelectFile:ap {w} {
    global saveimage

    $w.box invoke
}


# Procedure:	SelectFile:apply
#
# Usage:	SelectFile:apply widget filename
#
# Summary:	Apply callback: sends the contents of the dialog box to the
#		remote file id.
#
proc SelectFile:apply {w filename} {
    global selectfile rawimage FSLDIR

    switch $selectfile($w,funcname) {

	"OpenFolder" {
	    Message:InvokeFunction OpenFolder "FileName ($filename)"
	}

	"SaveFolderAs" {
	    if { ! [ catch { set cookie $selectfile($w,cookie) } ] } {
		Message:InvokeFunction SaveFolder \
		    "FileName ($filename) Folder $cookie"
	    } else {
		Message:InvokeFunction SaveFolder \
		    "FileName ($filename)"
	    }
	}

	"SaveImageAs"  {
	    if { $selectfile($w,format) == "AutoDetectFormat" } {
		set format ""
	    } else {
		set format "FileFormat $selectfile($w,format)"
	    }

	    Message:InvokeFunction SaveImage "
                FileName ($filename)
		$format
	    "
	}

	"SaveGroupAs"  {
	    set format "FileFormat $selectfile($w,format)"
	    Message:InvokeFunction Save3DGroup "
                FileName ($filename)
		$format
	    "
	}

	"ReadEPSF" {
	    Message:InvokeFunction ImportEPSGraphic "
		FileName ($filename) "
	}

	"ApplyParadigm" {
	    Message:InvokeFunction ApplyParadigmFile "
		/FileName	($filename)"
	}

	"ApplyShadowTransform" {
	    Message:InvokeFunction ApplyShadowTransform "
		/transformsource	($filename)
		/transformation		$selectfile($w,format)"
	}

	"SaveShadowTransform" {
	    Message:InvokeFunction SaveShadowTransform "
		/filename		($filename)"
            catch { exec sh -c " ${FSLDIR}/bin/fixxfm $filename" & } ErrMsg
	}

	"SelectHeadHatFile" {
	    Message:InvokeFunction SelectHeadHat "
		/FileName		($filename)"
	}

	default {
#	    puts "SelectFile:apply what function $selectfile($w,funcname)?"
	}
    }
    SetHistoryProperties [ $w.box subwidget filter subwidget listbox ] \
        Session $selectfile(history)
}


# Procedure:	SelectFile:format
#
# Usage:	SelectFile:format widget
#
# Summary:	
#
proc SelectFile:format {w format} {
    global selectfile
    
    set selectfile($w,format) $format

    if {$format == "RawGeneric"} {
	[$w.format subwidget label] config -width 10
	pack $w.format $w.rawspecs -in $w.ff -side left
    } else {
	pack forget $w.rawspecs
	[$w.format subwidget label] config -width 16
	pack $w.format -in $w.ff -side top
    }
}


# Procedure:	SelectFile:destroy
#
# Usage:	SelectFile:destroy widget
#
# Summary:	Called whenever the widget is to be destroyed.
#		Unsets all global variables associated with the widget
#		before destroying it.
#
proc SelectFile:destroy {w} {
    global selectfile
    global saveimagedir
    global saveimagelist
    global oldsaveformat oldsavegformat oldtransform currenttype



    set saveimagedir [[[$w.box subwidget filter] subwidget entry] get]
#    set saveimagedir [string trimright $saveimagedir "*"]
#    set saveimagedir [string trimright $saveimagedir "/"]

    set histlist [[$w.box subwidget filter] subwidget listbox]
    set saveimagelist [GetListItems $histlist]

    Message:DialogUnposted $w

    if {$currenttype == "Save Image As"} {
	set oldsaveformat $selectfile($w,format)
    } elseif {$currenttype == "Save Group As"} {
	set oldsavegformat $selectfile($w,format)
    } elseif {$currenttype == "Apply Shadow Transform"} {
	set oldtransform $selectfile($w,format)
    }
    unset selectfile($w,format)
    unset selectfile($w,disposition)
    catch { unset selectfile($w,cookie) }
    destroy $w
}


# This "if" statement makes it possible to run this script file inside or
# outside of the main demo program "phase?".
#
if {![info exists phase_demo_running]} {
    lappend auto_path ../
    lappend auto_path ./
    wm withdraw .
    set bitmapdir ../bitmaps
    set remote_out stdout
    OpenFolder .fi2 
}

