/*  paradigm.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#if !defined(__paradigm_h)
#define __paradigm_h

#include <iostream>
#include <fstream>
#include "newmatap.h"
#include "newmatio.h"
#include <string>

using namespace NEWMAT;
namespace FILM {
  
  class Paradigm
    {
    public:

      Paradigm() :
	designMatrix(),
	tcontrasts(0,0),
	fcontrasts(0,0)
	{}
 
      const Paradigm& operator=(Paradigm& par)
      {
	designMatrix = par.designMatrix;
	tcontrasts = par.tcontrasts;
	fcontrasts = par.fcontrasts;
	return *this;
      }
	
      Paradigm(Paradigm& par) { operator=(par); }

      // getters and setters:
      Matrix& getDesignMatrix() { return designMatrix; }
      void setDesignMatrix(const Matrix& pdesignMatrix) { designMatrix = pdesignMatrix; }

      Matrix& getTContrasts() { return tcontrasts; }
      Matrix& getFContrasts() { return fcontrasts; }
      //void setContrasts(const Matrix& pcontrasts) { contrasts = pcontrasts; }

      // useful functions:
      void load(const string& p_paradfname, const string& p_tcontrastfname, const string& p_contrastfname, bool p_blockdesign, int p_sizets);

      static void read_vest_waveform(string p_fname, Matrix& p_mat);

      ~Paradigm() {}
 
    private:
      
      Matrix designMatrix;
      Matrix tcontrasts;
      Matrix fcontrasts;
    };

}

#endif
