#include <stdlib.h>
#include <stdio.h>
#include "avwio.h"

size_t AvwWriteVolumes(AVW *avw, const void *buffer, size_t nvols)
{
  size_t volbytes;
  size_t n;
  short type;

  volbytes = AvwGetVolSize(avw) * (AvwGetDataType(avw, &type) / 
				   (8 * sizeof(char)));

  if(!avw->sameorder)
    AvwReverseBuffer(avw, (void *)buffer, AvwGetVolSize(avw) * nvols);

  n = fwrite(buffer, volbytes, nvols, avw->imgfp);
  if(n != nvols)
    perror("AvwWriteVolumes");

  if(!avw->sameorder)		/* Reverse again so its still valid */
    AvwReverseBuffer(avw, (void *)buffer, AvwGetVolSize(avw) * nvols);

  return n;
}

size_t AvwWriteCplxVolumes(AVW *avw, void *buffer, size_t nvols, char mode)
{
  size_t volsize,bytes;
  size_t n;
  short type;

  AvwGetDataType(avw, &type);
  if(type != DT_COMPLEX) 
    perror("AvwWriteCplxVolumes");
  
  if((mode!='i')&&(mode!='r'))
    perror("AvwWriteCplxVolumes");
  
  volsize = AvwGetVolSize(avw);
  bytes = AvwGetDataType(avw, &type)/(8*sizeof(char)*2);

  if(!avw->sameorder){
    AvwSetDataType(avw,DT_FLOAT);
    AvwReverseBuffer(avw, (void *)buffer, AvwGetVolSize(avw) * nvols);
    AvwSetDataType(avw,DT_COMPLEX);
  }
  
  if (mode == 'i') {
    fseek(avw->imgfp, bytes, SEEK_CUR);
  }

  for (n=0; n<volsize*nvols; n++) {
    if((fwrite((char *)buffer+n*bytes, bytes, 1, avw->imgfp))!=1)
      perror("AvwWriteCplxVolumes");
    fseek(avw->imgfp, bytes, SEEK_CUR);
  }

  if (mode == 'i') {
    fseek(avw->imgfp, -bytes, SEEK_CUR);
  }

  if(!avw->sameorder){
    AvwSetDataType(avw,DT_FLOAT);
    AvwReverseBuffer(avw, (void *)buffer, AvwGetVolSize(avw) * nvols);
    AvwSetDataType(avw,DT_COMPLEX);
  }

  return nvols;
}

size_t AvwWriteSliceSeries(AVW *avw, const void *buffer,short slice, size_t nvols)
{
  size_t slbytes,volbytes;
  size_t n;
  short x,y,z,v,type;

  AvwGetDim(avw,&x,&y,&z,&v);
  
  if(slice>=z)perror("AvwWriteSlices");

  slbytes = x * y * (AvwGetDataType(avw, &type) / 
		     (8 * sizeof(char)));
  volbytes = slbytes * z;

  if(!avw->sameorder)
    AvwReverseBuffer(avw, (void *)buffer, x * y * nvols);

  fseek(avw->imgfp, slbytes*slice, SEEK_CUR);

  for(n=0; n<nvols; n++){
    if(n>0)fseek(avw->imgfp, volbytes - slbytes, SEEK_CUR);
    if(fwrite((char *)buffer+n*slbytes, slbytes, 1, avw->imgfp) != 1)
      perror("AvwWriteSlices");
  }

  fseek(avw->imgfp, slbytes*(z-slice-1), SEEK_CUR);

  if(!avw->sameorder)
    AvwReverseBuffer(avw, (void *)buffer, x * y * nvols);

  return n;
}
size_t AvwWriteRowSeries(AVW *avw, const void *buffer, short row, short slice, size_t nvols)
{
  size_t rowbytes,slbytes,volbytes;
  size_t n;
  short x,y,z,v,type;

  AvwGetDim(avw,&x,&y,&z,&v);

  if(slice>=z)perror("AvwWriteRows");
  if(row>=y)perror("AvwWriteRows");
  
  rowbytes = x * (AvwGetDataType(avw, &type) / 
		  (8 * sizeof(char)));
  slbytes = rowbytes * y;
  volbytes = slbytes * z;

  if(!avw->sameorder)
    AvwReverseBuffer(avw, (void *)buffer, y * nvols);

  fseek(avw->imgfp, rowbytes*row + slbytes*slice, SEEK_CUR);

  for(n=0; n<nvols; n++){
    if(n>0)fseek(avw->imgfp, volbytes - rowbytes, SEEK_CUR);
    if(fwrite((char *)buffer+n*rowbytes, rowbytes, 1, avw->imgfp) != 1)
      perror("AvwWriteRows");
  }

  fseek(avw->imgfp, (slbytes*(z-slice-1)) + ((y-row-1)*rowbytes), SEEK_CUR);

  if(!avw->sameorder)
    AvwReverseBuffer(avw, (void *)buffer, y * nvols);

  return n;

}
