/*  gaussComparer.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#if !defined(__gaussComparer_h)
#define __gaussComparer_h

#include <iostream>
#include <fstream>
#define WANT_STREAM
#define WANT_MATH

#include "histogram.h"
#include "newmatap.h"
#include "newmatio.h"

extern "C"
{
#include "libprob.h"
};

using namespace NEWMAT;

namespace FILM {
 
  class GaussComparer
    {
    public:
      GaussComparer(ColumnVector& pzscores) 
	: zscores(pzscores), histogram(zscores, -10.0F, 10.0F, 2000) {}
    
      void setup(){histogram.generate();} // sets up histogram
     
      float computeRatio(const float prob, ofstream& out); // returns ratio of tail size in zscores compared to a Gaussian distribution

    private:
      GaussComparer();
      const GaussComparer& operator=(GaussComparer&);
      GaussComparer(GaussComparer&);
     
      const ColumnVector& zscores;

      Histogram histogram;
    };

}

#endif




