/*  displacement.cc - Artificial Motion Generator
    
    Peter Bannister, FMRIB Image Analysis Group
    
    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */


#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <iomanip>
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <strstream>

#include "mjimage.h"
#include "generalio.h"
#include "interpolation.h"
#include "miscimfns.h"
#include "optimise.h"
#include "costfns.h"
#include "newmatap.h"

#include "Globaloptions.h"
#include "Log.h"

using namespace MISCMATHS;
using namespace MISCIMFNS;
using namespace INTERPOLATION;
using namespace COSTFNS;
using namespace NEWMAT;
using namespace MJIMAGE;
using namespace GENERALIO;
using namespace UTILS;

int main (int argc,char** argv){
  
  tvolume timeseries;
  volume epi;
  Matrix aff(4,4);
  string outputfname;
  Log& logger = Log::getInstance();
  logger.setDir("/usr/people/prb/medx/motion/validation/fmri_4.4.mat");

  read_volume(epi, "/usr/local/fdl/structural/spm99/templates/EPI.img");
  volume save(64,64,21);
  save. setvoxelsize(4,4,6);
 
  for (int i = 0; i < 180; i++){ 
    char strc[50];
    ostrstream osc(strc,50);
    osc << "MAT_0" << setw(3) << setfill('0') << i << '\0';
    cerr << "Applying matrix: " << strc << endl;
    logger. in(strc, aff,4,4);
    cerr << "read in matrix" << endl;
    cerr << aff << endl;
    filled_affine_transform(save, epi, aff);
    cerr << "transform applied ... " << endl;
    outputfname = string("vol_") + strc;
    save_volume(save, outputfname);
  }

}








