/*  MELODIC - Multivariate exploratory linear optimized decomposition into 
              independent components
    
    melodicpca.h - PCA and whitening 

    Christian F. Beckmann, FMRIB Image Analysis Group
    
    Copyright (C) 1999-2001 University of Oxford */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#include "newimageall.h"
#include "Log.h"
#include "melodicoptions.h"
#include "melodicdata.h"
#include "melodic.h"
#include "newmatap.h"
#include "newmatio.h"
#include "melodicpca.h"

using namespace Utilities;
using namespace NEWIMAGE;

namespace Melodic{
      
   void MelodicPCA::perf_pca(const Matrix &Data)
   {    
     message("Starting PCA  ... ");

     SymmetricMatrix Corr;

     if(!opts.segment.value()){
     Corr = cov(Data.t());
     }
     else{
       message("  using gray-matter segmentation for covariance matrix " << endl);
       Matrix Res;
       Res = ones(Data.Nrows(),1)*melodat.get_GMseg();
       Res = SP(Data,Res);
       melodat.set_data(Res);
       Corr = cov(Res.t());
     }

     Matrix tmpE;
     DiagonalMatrix tmpD;
 
     EigenValues(Corr,tmpD,tmpE);

     if(opts.tsmooth.value()){
       message(" temporal smoothing of the Eigenvectors " << endl);
       tmpE=melodat.smoothColumns(tmpE);
     }
  
     melodat.set_pcaE(tmpE);
     melodat.set_pcaD(tmpD);

     message("done" << endl);
   }
  
  void MelodicPCA::perf_white(const Matrix &Data)
  {
    Matrix RE;
    DiagonalMatrix RD;
    Matrix tmpWhite;
    Matrix tmpDeWhite;

    int N = melodat.get_pcaE().Ncols();    
    if(opts.pca_dim.value() > N){
      cerr << "wrong dimensionality - using -dim " << N << 
              " instead " << endl;
      opts.pca_dim.set_T(N);
    }
    if(opts.pca_dim.value() < 1){
      if(opts.remove_meanvol.value()){
	opts.pca_dim.set_T(N-1);
      }else{
	opts.pca_dim.set_T(N);
      }
    }
    if(opts.approach.value()==string("jade") && opts.pca_dim.value() > 30){
      cerr << "dimensionality too large for jade estimation - using --dim 30 instead" << endl;
      opts.pca_dim.set_T(30);
    }
    message("Start whitening using  "<< opts.pca_dim.value()<<" dimensions ... ");

    RE = melodat.get_pcaE().Columns(N-opts.pca_dim.value()+1,N);
    RD << abs(melodat.get_pcaD().SymSubMatrix(N-opts.pca_dim.value()+1,N));    

    tmpWhite = sqrt(abs(RD.i()))*RE.t();
    tmpDeWhite = RE*sqrt(RD);
    melodat.set_white(tmpWhite);
    melodat.set_dewhite(tmpDeWhite);
    message("done"<< endl << endl);
   }
}


