#

# cut_corner
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

# this prompts for a 2D rectangle and then deletes inside this and all
# corresponding places in all higher slices

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc cut_corner {Folder Volume Popups} {

# find number of slices for use at end
MxGetImageProperties $Volume VolProps
set VolZ [keylget VolProps Slices]

# set graphics mode to 2D and prompt user to draw rectangle
MxSetLightboxView $Volume
MxSetImageOpMode $Volume 2D1
MxSetGraphicToolMode Rectangle
MxPause "Please draw a rectangle"

# grab rectangle and get its properties
MxGetCurrentGraphic G0
MxSetImageOpMode $Volume 3D
MxGetRectangleProperties $G0 Properties
set x [keylget Properties X]
set y [keylget Properties Y]
set width [keylget Properties Width]
set height [keylget Properties Height]
set z [keylget Properties Z]
set depth [expr $VolZ - $z]

# find image value to set inside rectangle
MxGetDisplayRange $Volume V themin themax

# draw 3D box derived from 2D rectangle and delete inside
MxDrawBox $Volume $x $y $z $width $height $depth G1
MxExpr "{EvalROI $G1}" $Volume = " $themin "

# remove graphics
MxDeleteGraphic { $G0 }
MxDeleteGraphic { $G1 }

# reset display range
MxSetDisplayRange $Volume V $themin $themax

}
