#

#   mask - convert image to MINC and run Display and the reread image
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc mni_mask {Folder In TheArgs} {
    global PXHOME USER PROCID exceed

# save image to temp AVW file
FSLSaveAs $In AVW /tmp/${USER}_${PROCID}_mask.hdr true

# convert to minc
set thecommand "exec /usr/local/mni/bin/analyze2minc /tmp/${USER}_${PROCID}_mask"
ScriptUpdate "Converting to MINC format: $thecommand"
set result [catch {eval $thecommand} ErrMsg]

# run external program
set thecommand "exec xterm -e /usr/local/mni/bin/Display /tmp/${USER}_${PROCID}_mask.mnc &"
set result [catch {eval $thecommand} ErrMsg]

after 5000
after 5000
MxPause "Now create a coloured mask in the \"Display\" program from Montreal. DO NOT CHANGE THE DEFAULT LABEL COLOUR/VALUE! Use right-mouse to draw mask points and shift-right-mouse to delete mask points. Use left- and middle-mouse to move around in the image. To change the mask drawing cursor size click (in the menu window) \"Segmenting\", then \"XY Radius\", and enter the value in the XTerm, and finally press \"Pop Menu\". When you are finished, save the mask to file using (in the menu window) \"File\", then \"Save Labels .mnc\" and enter the following filename in the XTerm and press return: /tmp/${PROCID}.mnc then you can exit \"Display\"\n\nWhen you are finished press OK"

# convert to minc
set thecommand "exec /usr/local/mni/bin/mincresample /tmp/${PROCID}.mnc /tmp/${USER}_${PROCID}_label.mnc -like /tmp/${USER}_${PROCID}_mask.mnc"
ScriptUpdate "Converting to AVW format 1/2: $thecommand"
set result [catch {eval $thecommand} ErrMsg]
set thecommand "exec /usr/local/mni/bin/minc2analyze /tmp/${USER}_${PROCID}_label"
ScriptUpdate "Converting to AVW format 2/2: $thecommand"
set result [catch {eval $thecommand} ErrMsg]

MxOpenImage $Folder /tmp/${USER}_${PROCID}_label.hdr I1
MxSetCurrentPage $I1

}
