#include "filebmp.h"
using namespace std;


#ifdef _WINDOWS
typedef BITMAPFILEHEADER BITMAP_FILE_HEADER;
#else 
struct BITMAP_FILE_HEADER 
{
        WORD    bfType;			//Specifies the file type. It must be BM. 
        UINT    bfSize;			//Specifies the size, in bytes, of the bitmap file. 
        WORD    bfReserved1;	//Reserved; must be zero. 
        WORD    bfReserved2;	//Reserved; must be zero. 
        UINT    bfOffBits;		//Specifies the offset, in bytes, from the BITMAPFILEHEADER structure to the bitmap bits. 
};
#endif

/////////////////////////////////////////////////////////////////////
/// ZBMPImage

ZImageBase* ZBMPImage::ReadFile()
{
	if(!IsOpen())
	{
		ZWarning("BMPReader", "%s -- File is not opened!", m_filename.c_str());
		return 0;
	}

	BITMAP_FILE_HEADER bmfHeader;
	BITMAPINFOHEADER bmpInfoHeader;
	PBYTE			pImageBuf;
	PALETTE_ENTRY*	pal=0;
	UINT	Width, Height, aWidth, dwBitsSize;

	/*
	 * get length of DIB in bytes for use when reading
	 */

	m_ImageStream.seekg(0, ios::end);
	dwBitsSize = m_ImageStream.tellg();
	m_ImageStream.seekg(0, ios::beg);


	/*
	 * Go read the DIB m_file header and check if it's valid.
	 */
	Read2((char*)(&(bmfHeader.bfType)));
	Read4((char*)(&(bmfHeader.bfSize)));
	Read2((char*)(&(bmfHeader.bfReserved1)));
	Read2((char*)(&(bmfHeader.bfReserved2)));
	Read4((char*)(&(bmfHeader.bfOffBits)));
	if(Fail())
	{
		ZWarning("BMPReader", "%s -- Can not read the BMP header!", m_filename.c_str());
		return 0;
	}

	if (bmfHeader.bfType != DIB_HEADER_MARKER)
	{
		ZWarning("BMPReader", "%s -- Not a valia BMP file!", m_filename.c_str());
		return 0;
	}

	/*
	 * Read the Bitmap InfoHead
	 */
	Read4((char*)&(bmpInfoHeader.biSize), 3);
	Read2((char*)&(bmpInfoHeader.biPlanes), 2);
	Read4((char*)&(bmpInfoHeader.biCompression), 6);
	if(Fail())
	{
		ZWarning("BMPReader", "%s -- Can not read the BMP Infoheader!", m_filename.c_str());
		return 0;
	}

	Width = bmpInfoHeader.biWidth;
	Height = bmpInfoHeader.biHeight;
	UINT numColors;

	bool	bGray = true;
	aWidth = (Width * bmpInfoHeader.biBitCount + 31) /32 * 4;
	numColors = bmpInfoHeader.biBitCount > 8 ? 0 : short(1) << bmpInfoHeader.biBitCount;

	ZImagePalette palette;
	if(numColors>0)
	{
		if(bmpInfoHeader.biClrUsed == 0) bmpInfoHeader.biClrUsed = numColors;
		pal = new PALETTE_ENTRY[bmpInfoHeader.biClrUsed];
		Read((char*)pal, bmpInfoHeader.biClrUsed * sizeof(PALETTE_ENTRY));
		if(Fail())
		{
			ZWarning("BMPReader", "%s -- Can not read the BMP Infoheader!", m_filename.c_str());
			delete[]pal;
			return 0;
		}
		palette.Create(bmpInfoHeader.biClrUsed, pal);
		delete []pal;
	}

	if(numColors > 2)
	for(UINT i=0; i<bmpInfoHeader.biClrUsed; i++)
	{
		UINT rgb = palette.GetRGB(i);
		if(GetR(rgb) != GetG(rgb) || GetR(rgb) != GetB(rgb))
		{
			bGray = false;
			break;
		}
	}

	pImageBuf = new BYTE[aWidth*Height];

	/*
	 * Go read the bits.
	 */
	for(int i = Height-1; i > -1; i--)
	{
		if(!Read(pImageBuf + i * aWidth, aWidth))
		{
			ZWarning("BMPReader", "%s -- Error occurs when reading!", m_filename.c_str());
			break;
		}
	}

	if(numColors == 0)
	{
		int BytesPerPixels = bmpInfoHeader.biBitCount == 24 ? 3 : 4;
		PBYTE ptr = pImageBuf;
		UINT width = 3 * Width;
		for(UINT j=0; j<Height; j++, ptr+=aWidth)
		for(UINT i=0; i<width; i+=BytesPerPixels) swap(ptr[i], ptr[i+2]);
	}

	ZImageBase*	image = CreateImageFromIndex(pImageBuf, Width, Height, aWidth, palette, bmpInfoHeader.biBitCount);

	delete []pImageBuf;
	
	return image;
}

bool ZBMPImage::WriteFile(const ZImageBase& image, bool intensityscale)
{
	if(!IsOpen())
	{
		ZWarning("BMPWriter", "%s -- File is not opened!", m_filename.c_str());
		return false;
	}

	if(!image.IfValid() || image.GetBuffer() == 0)
	{
		ZWarning("BMPWriter", "%s -- Image buffer not available!", m_filename.c_str());
		return false;
	}

	const ZImageBase*	pimage = &image;
	bool			freeflag = false;

	if(image.Depth() > 1) image.GotoSlice(0);

	if((image.PixFmt() & epixfmtNumericTypeMask) != epixfmtChar || intensityscale) 
	{
		ZImageBase*	ppimage;
		freeflag = true;
		if(image.isColor())
		{
			ppimage = new ZRGBByteImage;
			image.CopyPixelsTo(*ppimage, 0, 255);
		}
		else
		{
			ppimage = new ZGrayByteImage;
			image.CopyPixelsTo(*ppimage, 0, 255);
		}
		pimage = ppimage;
	}

	BITMAP_FILE_HEADER	bmfHdr;

	UINT bmpFileHeadSize = 14;//sizeof(BITMAP_FILE_HEADER);
	UINT bmpInfoHeadSize = sizeof(BITMAPINFOHEADER);
	UINT Width = image.Width(), awidth;
	UINT Height = image.Height();
	UINT ncolors = 0;
	
	int PixelSize = 3;
	if(image.isColor())	awidth = (Width * 24 + 31) /32 * 4;
	else
	{
		awidth = (Width + 3) /4 *4 ;
		ncolors = 256, PixelSize = 1;
	}

	/* Fill in file type (first 2 bytes must be "BM" for a bitmap) */
	bmfHdr.bfType = DIB_HEADER_MARKER;  // "BM"

	UINT bmpInfoSize = bmpInfoHeadSize + sizeof(PALETTE_ENTRY) * ncolors;

	// Now calculate the size of the image

		// Now, since we have calculated the correct size, why don't we
		// fill in the biSizeImage field (this will fix any .BMP files which
		// have this field incorrect).

	// Calculate the file size by adding the DIB size to sizeof(BITMAPFILEHEADER)

	bmfHdr.bfSize = Width * Height + bmpInfoSize + bmpFileHeadSize;
	bmfHdr.bfReserved1 = 0;
	bmfHdr.bfReserved2 = 0;

	/*
	 * Now, calculate the offset the actual bitmap bits will be in
	 * the file -- It's the Bitmap file header plus the DIB header,
	 * plus the size of the color table.
	 */
	bmfHdr.bfOffBits = (UINT)bmpFileHeadSize + bmpInfoSize;

	// Write the file header
	Write2((char*)(&(bmfHdr.bfType)));
	Write4((char*)(&(bmfHdr.bfSize)));
	Write2((char*)(&(bmfHdr.bfReserved1)));
	Write2((char*)(&(bmfHdr.bfReserved2)));
	Write4((char*)(&(bmfHdr.bfOffBits)));

	// Write the DIB header and the bits
	//
	BITMAPINFO* bmpInfo;
	image.FillBitmapInfo(&bmpInfo);
	bmpInfo->bmiHeader.biHeight = -bmpInfo->bmiHeader.biHeight;

	Write4((char*)&(bmpInfo->bmiHeader.biSize), 3);
	Write2((char*)&(bmpInfo->bmiHeader.biPlanes), 2);
	Write4((char*)&(bmpInfo->bmiHeader.biCompression), 6);
	Write((char*)bmpInfo->bmiColors, sizeof(PALETTE_ENTRY) * ncolors);

	if(Fail())
	{
		ZWarning("BMPWriter", "%s -- Error occurs when writing!", m_filename.c_str());
		return false;
	}

	ZImage<BYTE>			gimage;
	ZImage< ZRGB<BYTE> >	cimage;

	PBYTE src;
	if(image.isColor())	image.CopyPixelsTo(cimage), src = cimage.GetBuffer();
	else image.CopyPixelsTo(gimage), src = gimage.GetBuffer();

	PBYTE buf = new BYTE[awidth * image.Height()];
	PBYTE dst = buf;

	UINT j=0;
	for(; j<image.Height(); j++, src += image.Width()*PixelSize, dst+=awidth)
	{
		memcpy(dst, src, image.Width()*PixelSize);
	}

	dst = buf;
	if(ncolors == 0)
	{
		Width *= 3;
		PBYTE	ptr = dst;
		for(UINT j=0; j<Height; j++, ptr+=awidth)
		for(UINT i=0; i<Width; i+=3) swap(ptr[i], ptr[i+2]);
	}

	dst = buf;
	for(int k = Height-1; k >= 0; k--)
	{
		if(!Write(dst + k * awidth, awidth))
		{
			ZWarning("BMPWriter", "%s -- Error occurs when writing!", m_filename.c_str());
		}
	}
	delete []bmpInfo;
	delete []buf;
	
	if(freeflag) delete pimage;
	return true;
}
